
package calculette;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

class CalcException extends Exception {
	CalcException(String msg){
		super(msg);
	}
}

class Pile {
	static int MAXPILE=30;
	int sp=0;
	int[] st=new int[MAXPILE];

	boolean inNumber=false;

	void setInNumber(boolean b){inNumber=b;}
	boolean isInNumber() {return inNumber;}

	TextField viewer;

	Pile(TextField v){
		viewer = v;
	}

	int depiler() throws CalcException {
		if (sp<=0){
			throw(new CalcException("erreur : pile vide"));
		}
		else
			return(st[sp--]);
	}

	void empiler (int v) throws CalcException {
		if (sp>=MAXPILE){
			throw(new CalcException("erreur : pile pleine"));
		}
		st[++sp]=v;
	}

	void reset(){
		sp=0;
		st[sp]=0;
		setInNumber(false);
		display();
	}

	int sommet(){
		return(st[sp]);
	}

	void display(){

		if (viewer != null){
			viewer.setText(""+sommet());
		} else {
			System.out.println("sommet: " + sommet());
		}
		System.out.print("sp="+sp+" :: ");
		for (int i=0;i<=sp ;i++ ){
			System.out.print(st[i]+" ");
		}
		System.out.println();
	}
}

abstract class Operator {
	Pile pile;
	public void exec(){}

	Operator(Pile p){
		pile = p;
	}
}

class Chiffre extends Operator {
	int val=0;
	public void exec(){
		try{
			if (pile.isInNumber()){
				int a = pile.depiler();
				a=a*10+val;
				pile.empiler(a);
			} else {
				pile.empiler(val);
				pile.setInNumber(true);
			}
			pile.display();
		}catch (CalcException e){
			System.out.println(e.getMessage());
			pile.reset();
		}
	}
	Chiffre(Pile p, int v){
		super(p);
		val = v;
	}
}

abstract class OperateurArith extends Operator {
  public void exec(){
		try{
			int a = pile.depiler();
			int b = pile.depiler();
			pile.empiler(applique(b,a));
			pile.setInNumber(false);
			pile.display();
		}catch (CalcException e){
			System.out.println(e.getMessage());
			pile.reset();
		}
	}

	abstract int applique(int x, int y);

	OperateurArith(Pile p){
		super(p);
	}
}

class Plus extends OperateurArith {
	int applique(int x, int y){
		return(x+y);
	}

	Plus(Pile p){
		super(p);
	}
}

class Mult extends OperateurArith {

	int applique(int x, int y){
		return(x*y);
	}

	Mult(Pile p){
		super(p);
	}
}

class Minus extends OperateurArith {

	int applique(int x, int y){
		return(x-y);
	}

	Minus(Pile p){
		super(p);
	}
}

class Div extends OperateurArith {
	int applique(int x, int y){
		return(x/y);
	}

	Div(Pile p){
		super(p);
	}
}

class Enter extends Operator {
	int val=0;
	public void exec(){
			pile.setInNumber(false);
	}
	Enter(Pile p){
		super(p);
	}
}

class Clear extends Operator {
	int val=0;
	public void exec(){
			pile.reset();
	}
	Clear(Pile p){
		super(p);
	}
}

public class Calculette extends Frame
{
	public static void main(String[] args)
	{
		System.out.println("salut les copains");
		new Calculette();
	}

	public Calculette(){

	    addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

		TextField viewer=new TextField();
		Pile p=new Pile(viewer);

		setLayout(new BorderLayout());
		add(viewer,"North");
		Panel bPanel=new Panel(new GridLayout(4,3));

		for(int i=1;i<=9;i++){
			createChiffreButton(bPanel,p,i);
		}
		createChiffreButton(bPanel,p,0);

		add(bPanel,"Center");

		// Les operateurs
		Panel oPanel=new Panel(new GridLayout(6,1));
		createOpButton(oPanel, "Enter", new Enter(p));
		createOpButton(oPanel, "Plus", new Plus(p));
		createOpButton(oPanel, "Mult", new Mult(p));
		createOpButton(oPanel, "Minus", new Minus(p));
		createOpButton(oPanel, "Div", new Div(p));
		createOpButton(oPanel, "Clear", new Clear(p));
		add(oPanel,"East");
		p.display();

		setSize(300,400);
		show();
	}

	void createChiffreButton(Panel p, final Pile s, final int v){
		Button b=new Button(""+v);
		p.add(b);
		b.addActionListener(new ActionListener(){
				Chiffre ch = new Chiffre(s,v);
				public void actionPerformed(ActionEvent e){
					ch.exec();
				}
		});
	}

	void createOpButton(Panel p, String s, final Operator op){
		Button b=new Button(s);
		p.add(b);
		b.addActionListener(new ActionListener(){
				Operator o = op;
				public void actionPerformed(ActionEvent e){
					o.exec();
				}
		});
	}
}
